/*     Dfinitions diverses pour les modules d'analyse numrique */


/*     Constants defines */


#ifdef __TURBOC__
#ifdef __PASCAL__
#error Pascal calling conventions are not yet supported
#endif

#ifndef __LARGE__
#error Compilation should occur *ONLY* in LARGE Model
#endif
#endif






                /*      Error codes returned by functions       */

#define ENOERROR	  0   /* No error 				*/
#define ETOLLE0          -1   /* Tolerance lower than or equal to 0	*/
#define EOVERMAXITER     -2   /* Attempted more iterations than allowed */
#define ENULLSLOPE       -3   /* Function derivative is 0		*/
#define ENOPARAB3        -4   /* No parabola fitting three given points	*/	
#define WCPLXROOTS       -5   /* Possible complex roots			*/
#define EYSAMESIGN       -6   /* f(x1) and f(x2) same sign		*/ 	
#define EMAXITERLT0      -7   /* Max nubr of allowed iterations l.e. 0	*/
#define ENOPXINTERSECT   -8   /* No parabola intersect with x axis	*/
#define EDIMLT0          -9   /* Array dimension lower than 0		*/
#define EMATSING       	-10   /* Singular matrix			*/		
#define EDIMLE0        	-11   /* Array dimension lower than-equal to 0  */
#define ENOFITPARAB   	-12   /*            *** RESERVED ***            */
#define E0DIVIDE  	-13   /* Attempted 0 divide			*/
#define EDEGLT2		-14   /* Degree lower than or equal to 2	*/	
#define ENOWCORE	-15   /* Not enough working core		*/
#define EDEGLE0		-16   /* Degree lower than or equal to 0	*/	
#define ENUMINTLE0	-17   /* Number of intervals l.e. 0		*/
#define ELOGLE0		-18   /* Attempted compute log a negative real	*/
#define ENDPTSLT2	-19   /* One or less data points given (lsq)    */
#define ENTRMLT1	-20   /* Less than one term asked (lsq)		*/
#define ENDTPTLTNTRMS	-21   /* Less data points than terms asked (lsq)*/
#define ENOSOL		-22   /* No solution				*/
#define E0MATDIAG	-23   /* Matrix diagonal contains 0s		*/
#define WNODIAGDOM	-24   /* Not a diagonal dominant matrix		*/
#define EMATNOTSYM	-25   /* Not a symmetric matrix         	*/


/* Public types */


typedef struct complex COMPLEX;

typedef enum {expolsq, fourierlsq, loglsq, polylsq, powerlsq, xpowerlsq}
		lsqfit;

typedef enum {FALSE, TRUE} bool;


/*	Now the provided routines     */
/*	C-C++ compatibility hack      */

#ifdef __cplusplus
extern "C" {
#endif

/*      Functions in module FCOMPLEX  */





extern void conjugue(COMPLEX, COMPLEX *);

extern double modulus(COMPLEX);

extern double carg(COMPLEX);

extern void cadd(COMPLEX, COMPLEX, COMPLEX *);

extern void csub(COMPLEX, COMPLEX, COMPLEX *);

extern void cmult(COMPLEX, COMPLEX, COMPLEX *);

extern void rmult(double, COMPLEX *);

extern void cdiv(COMPLEX, COMPLEX, COMPLEX *, int *);

extern void cpow(COMPLEX, int, COMPLEX *);

extern void csqrt(COMPLEX, COMPLEX *);

extern void rassign(double, COMPLEX *);




/*      Functions in module RTESTS and CTESTS */



extern bool isnear0(double);

extern bool isinfinite(double x);

extern bool rtestroot(double, double, double, double);

extern bool ctestroot(COMPLEX, COMPLEX, COMPLEX, double);

extern void checkslope(double, int *);




/*      Equation solving functions      */



extern void newton_raphson(double, double,
			   int,
			   double *, double *, double *,
			   int *, int *,
			   double f(double), double fprime(double));

extern void bisect(double, double, double,
		   int,
		   double *, double *,
		   int    *, int *,
		   double f(double));



extern void secant(double, double, double,
		   int,
		   double *, double *,
		   int *, int *,
		   double f(double));


extern void newton_horner(int,
			  double *,
			  double, double,
			  int,
			  int *, int *,
			  double *, double *, double *,
			  double *, double *,
			  int *, int *);



extern void muller(COMPLEX ,
		double ,
		int ,
		COMPLEX *, COMPLEX *,
		int *, int *,COMPLEX f(COMPLEX));



extern void laguerre(int ,
	      COMPLEX *,
	      COMPLEX ,
	      double ,
	      int ,
	      int *,
	      COMPLEX *, COMPLEX *,
	      int *, int *);


extern void steffensen(double, double ,
		    int ,
		    double *, double *,
		    int *, int *,
		    double f(double));




/*      Elementary matrix routines      */



extern void swap_rows(int,
		      double *,
		      int, int ,
		      int *);

extern void rowmultadd(int ,
		       double *,
		       double ,
		       int , int );

extern void row_mat_div(int,
			double *,
			double,
			int,
			int *);

extern void slargest_vect(int,
			  double *,
			  double *);

extern void rdiv_vect(int,
		      double *,
		      double ,
		      int *);

extern void mult_mat_vect(int ,
			  double *, double *, double *,
			  int *);

extern double mult_vect_vect(int dim,
			     double *v1, double *v2);

extern void trans_size_system(int,
			      double *,
			      int,
			      double *m2);


extern bool ismatsym(int,
		     double *,
		     int *);

/*      Linear algebra functions        */



extern void determinant(int,
			double *, double *,
			int *);


extern void inverse(int,
		    double *,
		    double *,
		    int *);


extern void gauss_elimination(int ,
			      double *,
			      double *, double *,
			      int *);


extern void partial_pivot(int,
			  double *,
			  double *, double *,
			  int *);



extern void lu_decompose(int ,
			 double *, double *, double *,
			 int *);

extern void lu_solve(int ,
		     double *, double *, double *, double *vectsol,
		     int *);


extern void gauss_seidel(int ,
			 double *, double *,
			 double ,
			 int ,
			 double *,
			 int *, int *);




/*      Eigen values and vectors routines set   */


extern void eigen_power(int,
			double *,
			double *,
			int,
			double,
			double*, double *,
			int *, int *);

			/* This algorithm will not converge *
			 * if the approximate eigenvalue    *
			 * is orthogonal to the actual      *
			 * dominant one  		    */


extern void jacobi(int,
		   double *,
		   int ,
		   double,
		   double *, double *,
		   int *, int *);

extern void le_verrier(int,
		double *,
		double *,
		int *);

extern void eigen_vect(int,
		       double *,
		       int,
		       double *,
		       double *,
		       int *);

extern void householder_givens(int ,
			       double *,
			       int,
			       double *,
			       int *);


/*	Elementary real polynomial functions */

extern void rpoly2_solve(double , double , double ,
		  double *, double *,
			double *, double *);


extern void rpoly_value_deriv(int ,
		       double *,
		       double,
		       double *, double *);



extern void rpoly_one_root(int,
		    double *,
		    double , double ,
		    int ,
		    double *, double *, double *,
		    int *, int *);

extern void rpoly_reduce(int *,
		 double *,
		 double ,
		 int *);



/*   */

extern void simpson(double , double ,
		    int ,
		    double *,
		    int *,
		    double f(double));


extern void romberg(double, double, double,
		    int,
		    double *,
		    int *, int *,
		    double f(double));


/*	Least Squares routine	*/

void lsq(int ,
	 double *, double *,
	 int *,
	 double *, double *, double *,
	 double *, double *,
	 int *,
	 lsqfit );

extern char *lsqname(lsqfit);

#ifdef __cplusplus
}
#endif